/**
 * Title:        CoCompose
 * Copyright:    Copyright (c) 2002, 2003
 * Author:       Dennis Wagelaar
 * Organisation: University of Twente, Vrije Universiteit Brussel
 * Description:  CoCompose Design Language Tool
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License, version 2 as
 *  published by the Free Software Foundation.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  (See the file "COPYING" that is included with this source distribution.)
 */

package implementations;

import java.util.*;
import EDU.utwente.CoCompose.lang.*;
import EDU.utwente.CoCompose.translate.*;
import EDU.utwente.CoCompose.util.*;

/**
 * Implementation generator plugin class for implementing a Returns composite
 * in Java.
 *
 * @author Dennis Wagelaar
 * @version 0.1.06, 4 February 2003
 */

public class JavaReturns extends JavaImplementationGenerator
{
  //////////////////////////////////////////////////////////////////////////////
  // Constructors
  //////////////////////////////////////////////////////////////////////////////

  /**
   * Constructs a JavaReturns
   */
  public JavaReturns()
  {
  } //JavaReturns

  //////////////////////////////////////////////////////////////////////////////
  // Protected methods
  //////////////////////////////////////////////////////////////////////////////

  /**
   * Retrieves the Originator role of the composite.
   * @return The Originator CompositeRole.
   */
  protected CompositeRole getOriginatorRole()
  {
    return getComposite().getRole("Originator");
  } //getOriginatorRole

  /**
   * Retrieves the Result role of the composite.
   * @return The Result CompositeRole.
   */
  protected CompositeRole getResultRole()
  {
    return getComposite().getRole("Result");
  } //getResultRole

  /**
   * Checks the composite roles for existence.
   * @throws CoComposeException if a required role does not exist.
   */
  protected void checkRoles() throws CoComposeException
  {
    Composite composite = getComposite();
    if (getOriginatorRole() == null)
      throw new CoComposeException("Originator role not found in composite " +
        composite.getName(), composite);
    if (getResultRole() == null)
      throw new CoComposeException("Result role not found in composite " +
        composite.getName(), composite);
  } //checkRoles

  /**
   * Applies the implementation generator for each Originator concept.
   * @throws CoComposeException if generating implementation failed.
   */
  protected void applyPerOriginator() throws CoComposeException
  {
    Object link;
    Collection results = getConceptsFor(getResultRole());
    Iterator links = getModel().getLinkedNodes(getOriginatorRole()).iterator();
    while (links.hasNext())
    {
      link = links.next();
      if (link instanceof Concept)
        applyToOriginator((Concept) link, results);
    } //while
  } //applyPerOriginator

  /**
   * Checks whether the amount of Result concepts doesn't exceed 1.<BR>
   * PRE: results ::= Collectio&lt;Concept&gt;
   * @param results The collection of Result concepts.
   * @throws CoComposeException if the amount exceeds 1.
   */
  protected void checkResultsCount(Collection results)
    throws CoComposeException
  {
    if (results.size() > 1)
      throw new CoComposeException("Originator can only have one Result",
        getComposite());
  } //checkResultsCount

  /**
   * Applies the implementation generator to an Originator concept.
   *
   * @param originator The concept to apply the role part for.
   * @param results The set of result concepts.
   * @throws CoComposeException if generating implementation failed.
   */
  protected void applyToOriginator(Concept originator, Collection results)
    throws CoComposeException
  {
    Iterator resultIt;
    Concept result;
    String resultName, impl, form;
    int resultCount;

	checkResultsCount(results);
    impl = getCurrentImplementation(originator);
    impl = insertPrimitiveImplementations(impl, getPrimitives(results));
    // generate method return type
    resultIt = results.iterator();
    if (resultIt.hasNext())
    {
      result = (Concept) resultIt.next();
      impl = insertReturnType(impl, result.getName());
    }
    setCurrentImplementation(originator, impl);
  } //applyToOriginator

  //////////////////////////////////////////////////////////////////////////////
  // Public methods
  //////////////////////////////////////////////////////////////////////////////

  /**
   * Applies the implementation generator.
   *
   * @param implementation The implementation generator to apply.
   * @param generateImpl The language specific code generation operation
   *                     visitor.
   * @throws CoComposeException if generating implementation failed.
   */
  public void apply(ImplementationGenerator implementation,
    GenerateImplementationVisitor generateImpl) throws CoComposeException
  {
    super.apply(implementation, generateImpl);
    checkRoles();
    applyPerOriginator();
  } //apply
} //JavaReturns
