/**
 * Title:        CoCompose
 * Copyright:    Copyright (c) 2002, 2003
 * Author:       Dennis Wagelaar
 * Organisation: University of Twente, Vrije Universiteit Brussel
 * Description:  CoCompose Design Language Tool
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License, version 2 as
 *  published by the Free Software Foundation.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  (See the file "COPYING" that is included with this source distribution.)
 */

package translate;

import java.util.*;
import EDU.utwente.CoCompose.lang.*;
import EDU.utwente.CoCompose.util.*;

/**
 * Generates JAsCo language implementation for a CoCompose model.
 *
 * @author Dennis Wagelaar
 * @version 0.1.18, 22 November 2003
 */

public class JAsCoImplementationVisitor extends JavaImplementationVisitor
{
  //////////////////////////////////////////////////////////////////////////////
  // Constants
  //////////////////////////////////////////////////////////////////////////////

  private static final String J_ASPECTBEAN = "AspectBean";
  private static final String J_HOOK       = "Hook";
  private static final String J_CONNECTOR  = "Connector";

  //////////////////////////////////////////////////////////////////////////////
  // Constructors
  //////////////////////////////////////////////////////////////////////////////

  /**
   * Constructs a JAsCoImplementationVisitor.
   */
  public JAsCoImplementationVisitor()
  {
  } //JAsCoImplementationVisitor

  /**
   * Inserts the package name before each class.
   * @param impl the implementation to adapt
   * @param packageName the package name to insert
   * @return the adapted implementation
   * @throws CoComposeException on parse errors
   */
  private String insertPackageNames(String impl, String packageName)
    throws CoComposeException
  {
    int startIndex = impl.indexOf('{');
    if (startIndex == -1)
    {
      throw new CoComposeException("No {..} block found in " + 
        getClass().getName() + ".insertPackageNames(): " + impl);
    }
    // put package name in front of all 'leftmost' concept
    // references (e.g. <MyConcept>.<SetValue>())
    while ((startIndex = impl.indexOf(' ', startIndex) + 1) != 0)
    {
      char c = impl.charAt(startIndex);
      if ((c >= 'A') && (c <= 'Z'))
      { // keyword starts with uppercase, so assume classifier and paste in package name
        impl = impl.substring(0, startIndex) +
                  packageName + '.' + impl.substring(startIndex);
        startIndex += packageName.length() + 2;
      }
    }
    return impl;
  } //insertPackageNames

  //////////////////////////////////////////////////////////////////////////////
  // Protected methods
  //////////////////////////////////////////////////////////////////////////////

  /**
   * Implements the concept as a Class.
   * @param concept The concept to implement.
   * @param form The exact concept form.
   */
  protected void conceptAsClass(Concept concept, String form)
  {
    super.conceptAsClass(concept, form);
    String impl = getImpl(concept);
    impl = "package " + concept.getModel().getName() + ";\n" + impl;
    setImpl(concept, impl);
  } //conceptAsClass

  /**
   * Implements the concept as an Interface.
   * @param concept The concept to implement.
   */
  protected void conceptAsInterface(Concept concept)
  {
    super.conceptAsInterface(concept);
    String impl = getImpl(concept);
    impl = "package " + concept.getModel().getName() + ";\n" + impl;
    setImpl(concept, impl);
  } //conceptAsInterface

  /**
   * Implements the concept as an AspectBean.
   * @param concept The concept to implement.
   * @param form The exact concept form.
   */
  protected void conceptAsAspectBean(Concept concept, String form)
  {
    String impl;

    impl = "package " + concept.getModel().getName() + ";\n";
    impl += "/**\n * " + concept.getComments() + "\n */\n";
    impl += "class ";
    impl += concept.getName() + " {\n" +
            getImplementation(concept, form, true) + "}";
    setImpl(concept, impl);
    if (log != null)
      log.log("Implemented " + concept.getName() + " as " + form);
  } //conceptAsAspectBean

  /**
   * Implements the concept as a Hook.
   * @param concept The concept to implement.
   * @param form The exact concept form.
   */
  protected void conceptAsHook(Concept concept, String form)
  {
    String impl = new String();

    impl = "/**\n * " + concept.getComments() + "\n */\n";
    impl += "hook ";
    impl += concept.getName();
    if (form.startsWith("Abstract"))
      impl += ";";
    else
     impl += " {\n" + getImplementation(concept, form, true) + "}";
    setImpl(concept, impl);
    if (log != null)
      log.log("Implemented " + concept.getName() + " as " + form);
  } //conceptAsHook

  /**
   * Implements the concept as a Connector.
   * @param concept The concept to implement.
   * @param form The exact concept form.
   */
  protected void conceptAsConnector(Concept concept, String form)
  {
    String impl;

    impl = "/**\n * " + concept.getComments() + "\n */\n";
    impl += "static connector ";
    impl += concept.getName() + " {\n";
    impl += getImplementation(concept, form, true) + "}";
    setImpl(concept, impl);
    if (log != null)
      log.log("Implemented " + concept.getName() + " as " + form);
  } //conceptAsConnector

  /**
   * Writes the implementation of the model.
   * @param model The model to write the implementation for.
   * @param outdir The directory to save the implementation files.
   * @throws CoComposeException if writing the implementation failed.
   */
  protected void writeImplementation(Model model, String outdir)
    throws CoComposeException
  {
    Hashtable implementations = getImpls();
    Enumeration entities = implementations.keys();
    Concept concept;
    String impl, form;

    if (log != null)
    {
      log.log("Writing implementations to files");
      log.incLogLevel();
    }
    try
    {
      while (entities.hasMoreElements())
      { // no unique realms used in SEESCOA, so all entities are Concepts.
        concept = (Concept) entities.nextElement();
        form = concept.getForm();
        if (form.equals("AbstractClass") || form.equals("Class") ||
            form.equals("Interface"))
        { // write classes and interfaces to files
          impl = (String) implementations.get(concept);
          writeFile(outdir, concept.getName() + ".java", impl);
          if (log != null)
            log.log(concept.getName() + " written");
        }
        else if (concept.getForm().equals("AspectBean"))
        { // write aspectbeans to files
          impl = (String) implementations.get(concept);
          writeFile(outdir, concept.getName() + ".asp", impl);
          if (log != null)
            log.log(concept.getName() + " written");
        }
        else if (concept.getForm().equals("Connector"))
        { // write connectors to files
          impl = (String) implementations.get(concept);
          impl = insertPackageNames(impl, concept.getModel().getName());
          writeFile(outdir, concept.getName() + ".con", impl);
          if (log != null)
            log.log(concept.getName() + " written");
        }
      } //while
    }
    finally
    {
      log.decLogLevel();
    }
  } //writeImplementation

  //////////////////////////////////////////////////////////////////////////////
  // Public methods
  //////////////////////////////////////////////////////////////////////////////

  /**
   * Returns the target language this GenerateImplementationVisitor supports.
   *
   * @return The target language.
   */
  public String getTargetLanguage()
  {
    return "JAsCo";
  } //getTargetLanguage

  /**
   * Generates implementation for a Concept object.
   *
   * @param concept The concept to visit.
   * @throws CoComposeException if generating implementation failed.
   */
  public void visitConcept(Concept concept) throws CoComposeException
  {
    String form = concept.getForm();

    if (log != null)
    {
      log.log("Generating implementation for concept " + concept.getName());
      log.incLogLevel();
    }
    try
    {
      // now dispatch to specific code generators
      if (form.equals(J_ASPECTBEAN))
        conceptAsAspectBean(concept, form);
      else if (form.endsWith(J_HOOK))
        conceptAsHook(concept, form);
      else if (form.equals(J_CONNECTOR))
        conceptAsConnector(concept, form);
      else
        super.visitConcept(concept);
    }
    finally
    {
      if (log != null)
        log.decLogLevel();
    }
  } //visitConcept
} //class JAsCoImplementationVisitor